/**
*
* Automatically generated by the Whitchcraft Compiler Collection 0.0.1
*
* 07:20:25 Jul  1 2016
*
*/


/**
* Imported objects:
**/
extern void *version;
extern void *method_gsskeyex;
extern void *client_version_string;
extern void *session_id;
extern void *method_none;
extern void *pmonitor;
extern void *mon_dispatch_postauth20;
extern void *test_flag;
extern void *options;
extern void *devices;
extern void *no_port_forwarding_flag;
extern void *custom_environment;
extern void *readonly;
extern void *roaming_enabled;
extern void *dispatch;
extern void *num_listen_socks;
extern void *log_level;
extern void *pw;
extern void *mon_dispatch_proto15;
extern void *BSDopterr;
extern void *channel_pre;
extern void *ciphers;
extern void *authorized_principals;
extern void *oqueue;
extern void *oom_adjust;
extern void *method_hostbased;
extern void *mm_sshpam_device;
extern void *sensitive_data;
extern void *mon_dispatch;
extern void *outgoing_stream;
extern void *listen_socks;
extern void *resume_in_progress;
extern void *saved_argc;
extern void *deny_severity;
extern void *auth_debug_init;
extern void *handles;
extern void *log_stderr;
extern void *startup_pipes;
extern void *macs;
extern void *BSDoptind;
extern void *allow_severity;
extern void *umac_ctx;
extern void *config_file_name;
extern void *use_privsep;
extern void *child_state;
extern void *mon_dispatch_proto20;
extern void *saved_argv;
extern void *auth1_methods;
extern void *no_user_rc;
extern void *compat13;
extern void *session_id2_len;
extern void *method_passwd;
extern void *original_command;
extern void *iqueue;
extern void *authmethods;
extern void *loginmsg;
extern void *datafellows;
extern void *rexeced_flag;
extern void *channel_post;
extern void *rexec_argc;
extern void *key_is_cert_authority;
extern void *forced_command;
extern void *first_unused_handle;
extern void *privsep_pw;
extern void *xxx_kex;
extern void *_IO_stdin_used;
extern void *method_pubkey;
extern void *no_pty_flag;
extern void *utmp_len;
extern void *compat20;
extern void *rexec_argv;
extern void *no_daemon_flag;
extern void *gssapi_kerberos_mech;
extern void *server_version_string;
extern void *method_kbdint;
extern void *cfg;
extern void *current_keys;
extern void *sshpam_device;
extern void *the_authctxt;
extern void *inetd_flag;
extern void *no_agent_forwarding_flag;
extern void *incoming_stream;
extern void *num_handles;
extern void *BSDoptreset;
extern void *debug_flag;
extern void *mon_dispatch_postauth15;
extern void *forced_tun_device;
extern void *supported_mechs;
extern void *rexec_flag;
extern void *auth_debug;
extern void *no_x11_forwarding_flag;
extern void *client_addr;
extern void *method_gssapi;
extern void *BSDoptopt;
extern void *BSDoptarg;
extern void *gssapi_null_mech;
extern void *gss_kex_context;
extern void *startup_pipe;
extern void *session_id2;


/**
* Imported functions:
**/
void *kexgss_server();
void *get_hostkey_private_by_type();
void *evp_aes_128_ctr();
void *dispatch_protocol_error();
void *_init();
void *sys_tun_outfilter();
void *channel_input_open_confirmation();
void *__libc_csu_init();
void *session_close_by_channel();
void *channel_input_ieof();
void *evp_acss();
void *verbose();
void *channel_input_port_open();
void *kexdh_server();
void *channel_input_oclose();
void *channel_input_window_adjust();
void *packet_get_int();
void *channel_input_data();
void *kexgex_server();
void *key_equal();
void *__b64_pton();
void *channel_input_open_failure();
void *logit();
void *kex_input_kexinit();
void *_fini();
void *__libc_csu_fini();
void *kexecdh_server();
void *buffer_put_int();
void *channel_input_extended_data();
void *channel_input_close();
void *get_hostkey_index();
void *buffer_put_char();
void *dispatch_protocol_ignore();
void *sys_tun_infilter();
void *key_equal_public();
void *get_hostkey_public_by_type();
void *channel_input_close_confirmation();
void *packet_get_char();
void *__b64_ntop();

