/**
*
* Automatically generated by the Whitchcraft Compiler Collection 0.0.1
*
* 07:20:25 Jul  1 2016
*
*/


/**
* Imported objects
**/
extern void *ap_hack_apr_shm_pool_get;
extern void *ap_http_outerror_filter_handle;
extern void *ap_hack_apr_pool_abort_get;
extern void *ap_hack_ap_update_child_status_from_indexes;
extern void *ap_hack_apr_mcast_join;
extern void *ap_hack_apr_socket_sendto;
extern void *ap_hack_apr_md5_final;
extern void *ap_hack_ap_content_length_filter;
extern void *ap_hack_ap_hook_get_open_htaccess;
extern void *ap_hack_ap_limit_section;
extern void *ap_hack_apr_file_attrs_set;
extern void *ap_hack_apr_crypto_shutdown;
extern void *ap_hack_apr_allocator_mutex_set;
extern void *ap_hack_ap_exists_config_define;
extern void *ap_hack_ap_abort_on_oom;
extern void *ap_hack_ap_mpm_podx_signal;
extern void *unixd_module;
extern void *ap_hack_ap_get_server_protocol;
extern void *ap_hack_apr_pool_is_ancestor;
extern void *ap_hack_ap_expr_exec;
extern void *ap_hack_ap_run_access_checker_ex;
extern void *ap_hack_ap_fcgi_begin_request_body_to_array;
extern void *ap_hack_ap_str_toupper;
extern void *ap_hack_apr_hook_debug_show;
extern void *ap_preloaded_modules;
extern void *ap_hack_ap_update_child_status;
extern void *ap_hack_ap_setup_auth_internal;
extern void *ap_hack_apr_uid_name_get;
extern void *ap_hack_ap_unescape_url;
extern void *ap_hack_apr_thread_pool_thread_max_get;
extern void *ap_hack_ap_run_open_htaccess;
extern void *ap_hack_apr_dbm_close;
extern void *ap_hack_apr_pescape_path;
extern void *ap_hack_ap_hook_get_create_connection;
extern void *ap_hack_apr_proc_kill;
extern void *ap_hack_apr_dso_unload;
extern void *ap_hack_ap_uname2id;
extern void *ap_hack_apr_procattr_child_in_set;
extern void *ap_run_mode;
extern void *ap_core_input_filter_handle;
extern void *ap_hack_ap_save_brigade;
extern void *ap_hack_apr_thread_pool_top;
extern void *ap_hack_ap_expr_str_exec_re;
extern void *ap_hack_apr_socket_timeout_get;
extern void *ap_content_length_filter_handle;
extern void *ap_hack_apr_threadkey_private_get;
extern void *ap_hack_ap_run_optional_fn_retrieve;
extern void *ap_hack_ap_process_child_status;
extern void *ap_hack_ap_set_content_length;
extern void *ap_hack_apr_atomic_read32;
extern void *ap_hack_ap_hook_open_logs;
extern void *ap_hack_apr_pollcb_remove;
extern void *ap_hack_ap_pcfg_openfile;
extern void *ap_hack_apr_socket_type_get;
extern void *ap_hack_apr_dbd_transaction_start;
extern void *ap_hack_apr_pool_tag;
extern void *ap_multipart_boundary;
extern void *ap_hack_ap_escape_urlencoded;
extern void *ap_hack_ap_filter_flush;
extern void *ap_hack_ap_strchr;
extern void *ap_hack_ap_run_rewrite_args;
extern void *ap_hack_ap_should_client_block;
extern void *ap_hack_ap_run_drop_privileges;
extern void *ap_hack_apr_proc_create;
extern void *ap_hack_apr_random_insecure_ready;
extern void *ap_hack_apr_crypto_passphrase;
extern void *ap_hack_apr_shm_size_get;
extern void *ap_hack_apr_file_writev;
extern void *ap_hack_apr_memcache_hash;
extern void *ap_hack_ap_set_deprecated;
extern void *ap_hack_apr_brigade_flatten;
extern void *ap_hack_apr_proc_mutex_destroy;
extern void *ap_hack_ap_fini_vhost_config;
extern void *ap_hack_apr_is_empty_table;
extern void *ap_hack_ap_register_extra_mpm_process;
extern void *ap_hack_apr_mcast_loopback;
extern void *ap_hack_ap_find_token;
extern void *ap_hack_apr_socket_atreadeof;
extern void *ap_hack_ap_get_conn_module_loglevel;
extern void *ap_hack_apr_table_getm;
extern void *ap_hack_apr_bucket_eos_create;
extern void *ap_hack_apr_uid_current;
extern void *ap_hack_apr_dir_close;
extern void *ap_hack_apr_bucket_setaside_noop;
extern void *ap_hack_ap_rflush;
extern void *ap_hack_apr_allocator_owner_get;
extern void *ap_hack_apr_table_compress;
extern void *ap_hack_apr_md4_set_xlate;
extern void *ap_hack_apr_base64_encode;
extern void *ap_hack_ap_find_command;
extern void *ap_hack_apr_hash_count;
extern void *ap_hack_apr_punescape_entity;
extern void *ap_hack_apr_dir_make_recursive;
extern void *ap_hack_apr_xml_quote_string;
extern void *ap_hack_apr_atomic_xchgptr;
extern void *ap_max_requests_per_child;
extern void *ap_hack_ap_scan_script_header_err_brigade;
extern void *ap_hack_ap_find_module_name;
extern void *ap_hack_ap_log_perror_;
extern void *ap_hack_apr_table_merge;
extern void *ap_hack_apr_shm_remove;
extern void *ap_hack_apr_xml_parser_geterror;
extern void *ap_http_input_filter_handle;
extern void *ap_hack_apr_file_pipe_timeout_set;
extern void *ap_hack_ap_note_basic_auth_failure;
extern void *ap_hack_ap_scan_script_header_err_ex;
extern void *ap_hack_ap_scan_script_header_err_core_ex;
extern void *ap_hack_ap_process_resource_config;
extern void *ap_hack_ap_get_remote_host;
extern void *ap_hack_apr_memcache_create;
extern void *ap_hack_ap_hook_get_post_perdir_config;
extern void *ap_hack_apr_socket_sendv;
extern void *ap_hack_apr_atomic_xchg32;
extern void *ap_hack_apr_ipsubnet_create;
extern void *ap_hack_apr_hash_do;
extern void *ap_hack_ap_expr_exec_ctx;
extern void *ap_hack_ap_method_registry_init;
extern void *ap_hack_ap_pass_brigade;
extern void *ap_hack_ap_run_note_auth_failure;
extern void *ap_hack_apr_table_clear;
extern void *ap_hack_ap_set_sub_req_protocol;
extern void *ap_hack_apr_sockaddr_ip_get;
extern void *ap_hack_ap_index_of_response;
extern void *ap_hack_ap_remove_output_filter_byhandle;
extern void *ap_old_write_func;
extern void *ap_hack_apr_ldap_is_ldap_url;
extern void *ap_hack_apr_os_locale_encoding;
extern void *ap_hack_ap_hook_create_connection;
extern void *ap_hack_ap_run_child_init;
extern void *ap_hack_apr_rmm_calloc;
extern void *ap_hack_ap_set_string_slot_lower;
extern void *ap_hack_apr_mcast_hops;
extern void *ap_hack_apr_queue_push;
extern void *ap_hack_ap_varbuf_init;
extern void *ap_hack_ap_hook_auth_checker;
extern void *ap_hack_apr_skiplist_merge;
extern void *ap_hack_ap_add_per_dir_conf;
extern void *ap_hack_apr_crypto_block_encrypt;
extern void *ap_hack_apr_reslist_acquired_count;
extern void *ap_hack_apr_dbd_native_handle;
extern void *ap_hack_apr_strtoff;
extern void *ap_hack_ap_hook_pre_read_request;
extern void *ap_hack_apr_bucket_socket_make;
extern void *ap_hack_apr_memcache_find_server;
extern void *ap_hack_apr_memcache_set;
extern void *ap_hack_apr_time_now;
extern void *ap_hack_apr_temp_dir_get;
extern void *ap_hack_ap_replace_stderr_log;
extern void *ap_hack_apr_os_proc_mutex_get;
extern void *ap_hack_apr_xml_insert_uri;
extern void *ap_hack_ap_send_error_response;
extern void *ap_hack_ap_meets_conditions;
extern void *ap_hack_apr_table_setn;
extern void *ap_hack_ap_hook_get_mgmt_items;
extern void *ap_hack_apr_queue_trypop;
extern void *ap_hack_apr_random_after_fork;
extern void *ap_hack_ap_get_server_built;
extern void *ap_hack_apr_escape_entity;
extern void *ap_hack_apr_crypto_sha256_new;
extern void *ap_hack_ap_proc_mutex_create;
extern void *ap_hack_ap_is_url;
extern void *ap_hack_apr_pool_initialize;
extern void *ap_hack_apr_crypto_clear;
extern void *ap_hack_apr_array_make;
extern void *ap_hack_apr_file_open_flags_stdout;
extern void *ap_hack_apr_dbd_init;
extern void *logio_module;
extern void *ap_hack_apr_version_string;
extern void *ap_hack_ap_add_loaded_module;
extern void *ap_hack_apr_sha1_final;
extern void *ap_hack_apr_xml_parse_file;
extern void *ap_hack_apr_hash_this;
extern void *ap_hack_apr_base64_decode_binary;
extern void *ap_hack_apr_dbd_check_conn;
extern void *ap_hack_ap_basic_http_header;
extern void *ap_hack_ap_set_string_slot;
extern void *ap_hack_apr_hook_deregister_all;
extern void *ap_hack_ap_hook_check_config;
extern void *ap_hack_apr_threadkey_private_delete;
extern void *ap_hack_apr_socket_addr_get;
extern void *ap_hack_apr_uid_homepath_get;
extern void *ap_hack_apr_sha1_init;
extern void *ap_hack_ap_set_accept_ranges;
extern void *ap_hack_apr_filepath_get;
extern void *ap_hack_ap_hook_handler;
extern void *ap_hack_apr_dbd_pvquery;
extern void *ap_hack_apr_mcast_leave;
extern void *ap_hack_apr_dbd_open;
extern void *ap_hack_apr_hash_this_key_len;
extern void *ap_hack_apr_thread_rwlock_create;
extern void *ap_hack_apr_os_imp_time_get;
extern void *ap_hack_apr_os_proc_mutex_put;
extern void *ap_hack_apr_pool_userdata_setn;
extern void *ap_hack_apr_thread_rwlock_destroy;
extern void *ap_hack_apr_file_pool_get;
extern void *ap_hack_apr_random_secure_ready;
extern void *ap_hack_apr_thread_mutex_unlock;
extern void *ap_hack_ap_hook_get_get_mgmt_items;
extern void *ap_hack_apr_xml_empty_elem;
extern void *ap_hack_ap_fcgi_fill_in_header;
extern void *ap_hack_apr_bucket_immortal_create;
extern void *ap_hack_ap_hook_get_pre_connection;
extern void *ap_hack_ap_hook_get_mpm_query;
extern void *ap_hack_ap_open_piped_log_ex;
extern void *ap_hack_apr_dbd_error;
extern void *ap_hack_apr_bucket_heap_create;
extern void *ap_hack_apr_memcache_add_multget_key;
extern void *ap_hack_apr_thread_exit;
extern void *ap_hack_apr_skiplist_insert_compare;
extern void *ap__authz_ap_some_auth_required;
extern void *ap_hack_ap_field_noparam;
extern void *ap_hack_apu_version;
extern void *ap_hack_ap_mpm_podx_open;
extern void *ap_hack_ap_is_matchexp;
extern void *ap_hack_apr_escape_path;
extern void *ap_hack_apr_crypto_get_driver;
extern void *ap_hack_ap_context_document_root;
extern void *ap_bucket_type_error;
extern void *ap_extended_status;
extern void *ap_hack_ap_get_exec_line;
extern void *ap_hack_ap_hook_check_authn;
extern void *ap_hack_apr_brigade_writev;
extern void *ap_hack_apr_procattr_io_set;
extern void *ap_hack_apr_brigade_vprintf;
extern void *ap_hack_ap_resolve_env;
extern void *ap_hack_ap_hook_expr_lookup;
extern void *ap_hack_ap_hook_check_authz;
extern void *ap_hack_ap_get_loadavg;
extern void *ap_hack_apr_skiplist_remove_all;
extern void *ap_hack_ap_clear_auth_internal;
extern void *ap_hack_ap_find_child_by_pid;
extern void *ap_hack_apr_mmap_delete;
extern void *ap_hack_apr_os_imp_time_put;
extern void *ap_hack_ap_add_output_filter_handle;
extern void *ap_hack_apr_password_get;
extern void *ap_hack_apr_sdbm_delete;
extern void *ap_hack_apr_skiplist_previous;
extern void *ap_hack_apr_rmm_addr_get;
extern void *ap_hack_ap_mpm_pod_killpg;
extern void *ap_hack_ap_method_register;
extern void *ap_hack_apr_thread_pool_tasks_count;
extern void *ap_hack_apr_fnmatch;
extern void *ap_hack_apr_stat;
extern void *ap_hack_ap_satisfies;
extern void *ap_hack_apr_getopt_init;
extern void *ap_hack_ap_update_mtime;
extern void *ap_hack_ap_set_context_info;
extern void *ap_hack_apr_proc_mutex_lock;
extern void *ap_hack_ap_run_pre_mpm;
extern void *ap_hack_apr_bucket_setaside_notimpl;
extern void *ap_hack_apr_file_unlock;
extern void *ap_hack_apr_bucket_copy_notimpl;
extern void *ap_hack_apr_threadkey_data_set;
extern void *ap_hack_apr_dbd_set_dbname;
extern void *ap_hack_apr_poll;
extern void *ap_hack_ap_hook_get_insert_filter;
extern void *ap_hack_ap_note_digest_auth_failure;
extern void *ap_hack_ap_method_number_of;
extern void *ap_hack_ap_setup_make_content_type;
extern void *ap_hack_apr_thread_data_get;
extern void *ap_hack_apr_proc_other_child_register;
extern void *ap_hack_ap_recent_ctime;
extern void *ap_hack_ap_os_escape_path;
extern void *ap_hack_apr_uuid_parse;
extern void *ap_hack_ap_register_output_filter;
extern void *ap_hack_ap_set_content_type;
extern void *ap_hack_ap_set_module_config;
extern void *ap_hack_ap_make_dirstr_prefix;
extern void *ap_hack_apr_atoi64;
extern void *ap_ugly_hack;
extern void *ap_unixd_config;
extern void *ap_hack_ap_scan_script_header_err_strs_ex;
extern void *ap_hack_apr_memcache_incr;
extern void *ap_hack_apr_ldap_is_ldapi_url;
extern void *ap_hack_apr_array_clear;
extern void *ap_hack_ap_hook_header_parser;
extern void *ap_hack_ap_reset_module_loglevels;
extern void *ap_hack_apr_md4_final;
extern void *ap_hack_ap_find_etag_strong;
extern void *ap_hack_apr_os_default_encoding;
extern void *ap_hack_ap_register_auth_provider;
extern void *ap_hack_ap_make_dirstr_parent;
extern void *ap_hack_apr_base64_encode_binary;
extern void *ap_hack_ap_find_module_short_name;
extern void *ap_hack_apr_dbd_pvbquery;
extern void *ap_hack_apr_pescape_urlencoded;
extern void *ap_hack_ap_run_insert_filter;
extern void *ap_coredumpdir_configured;
extern void *ap_hack_apr_os_pipe_put;
extern void *ap_document_root_check;
extern void *ap_hack_ap_internal_redirect;
extern void *ap_hack_apr_memcache_find_server_hash;
extern void *ap_hack_apr_bucket_shared_make;
extern void *ap_hack_apr_xml_quote_elem;
extern void *ap_hack_apr_procattr_detach_set;
extern void *ap_hack_ap_send_mmap;
extern void *ap_hack_ap_hook_dirwalk_stat;
extern void *ap_hack_apr_thread_pool_tasks_cancel;
extern void *ap_hack_apr_filepath_list_split;
extern void *ap_scoreboard_image;
extern void *ap_hack_ap_location_walk;
extern void *ap_hack_ap_hook_get_drop_privileges;
extern void *ap_hack_ap_strrchr_c;
extern void *ap_hack_ap_some_authn_required;
extern void *ap_hack_ap_create_conn_config;
extern void *ap_hack_ap_lookup_provider;
extern void *ap_hack_ap_allow_overrides;
extern void *ap_hack_ap_condition_if_modified_since;
extern void *ap_hack_apr_table_do;
extern void *ap_hack_ap_hook_mpm_get_name;
extern void *ap_hack_apr_dbd_transaction_end;
extern void *ap_subreq_core_filter_handle;
extern void *ap_hack_apr_bucket_eos_make;
extern void *ap_hack_apr_thread_pool_scheduled_tasks_count;
extern void *ap_hack_apr_table_add;
extern void *ap_hack_apr_sockaddr_is_wildcard;
extern void *ap_hack_ap_hook_generate_log_id;
extern void *ap_hack_ap_run_sub_req;
extern void *ap_hack_ap_fcgi_header_to_array;
extern void *ap_hack_apr_thread_yield;
extern void *ap_hack_apr_global_mutex_destroy;
extern void *ap_hack_apr_gethostname;
extern void *ap_hack_apr_brigade_cleanup;
extern void *ap_hack_apr_sdbm_nextkey;
extern void *ap_hack_apr_escape_path_segment;
extern void *ap_hack_ap_send_http_options;
extern void *ap_hack_ap_escape_logitem;
extern void *ap_hack_apr_sdbm_rdonly;
extern void *ap_hack_ap_expr_str_exec;
extern void *ap_hack_ap_hook_insert_error_filter;
extern void *ap_hack_ap_set_file_slot;
extern void *ap_hack_apr_table_addn;
extern void *ap_hack_apr_xlate_conv_buffer;
extern void *ap_hack_ap_invoke_handler;
extern void *ap_hack_ap_remove_module;
extern void *ap_hack_ap_run_get_mgmt_items;
extern void *ap_hack_apr_crypto_block_encrypt_init;
extern void *ap_hack_ap_run_pre_connection;
extern void *ap_hack_ap_run_test_config;


/**
* Imported functions
**/
void *ap_str_tolower(void *arg1, ...);
void *ap_run_open_logs(void *arg1, ...);
void *ap_register_errorlog_handler(void *arg1, ...);
void *ap_hook_end_generation(void *arg1, ...);
void *ap_hook_fixups(void *arg1, ...);
void *ap_read_config(void *arg1, ...);
void *ap_hook_get_http_scheme(void *arg1, ...);
void *ap_unescape_urlencoded(void *arg1, ...);
void *ap_register_log_hooks(void *arg1, ...);
void *ap_get_remote_host(void *arg1, ...);
void *ap_hook_get_quick_handler(void *arg1, ...);
void *ap_set_receive_buffer_size(void *arg1, ...);
void *ap_hook_get_note_auth_failure(void *arg1, ...);
void *ap_hook_get_force_authn(void *arg1, ...);
void *ap_condition_if_modified_since(void *arg1, ...);
void *ap_expr_str_func_make(void *arg1, ...);
void *ap_close_selected_listeners(void *arg1, ...);
void *ap_get_core_module_config(void *arg1, ...);
void *ap_scan_script_header_err_strs_ex(void *arg1, ...);
void *ap_add_if_conf(void *arg1, ...);
void *ap_log_assert(void *arg1, ...);
void *ap_exists_config_define(void *arg1, ...);
void *ap_run_open_htaccess(void *arg1, ...);
void *ap_unixd_setup_child(void *arg1, ...);
void *ap_add_per_url_conf(void *arg1, ...);
void *ap_hook_map_to_storage(void *arg1, ...);
void *ap_run_translate_name(void *arg1, ...);
void *ap_hook_error_log(void *arg1, ...);
void *ap_soak_end_container(void *arg1, ...);
void *ap_set_config_vectors(void *arg1, ...);
void *ap_mpm_pod_check(void *arg1, ...);
void *ap_run_auth_checker(void *arg1, ...);
void *ap_vhost_iterate_given_conn(void *arg1, ...);
void *ap_add_module(void *arg1, ...);
void *ap_run_process_connection(void *arg1, ...);
void *ap_log_command_line(void *arg1, ...);
void *ap_escape_urlencoded(void *arg1, ...);
void *ap_get_limit_req_body(void *arg1, ...);
void *ap_if_walk(void *arg1, ...);
void *ap_set_string_slot(void *arg1, ...);
void *ap_mpm_podx_close(void *arg1, ...);
void *ap_process_child_status(void *arg1, ...);
void *ap_strcasecmp_match(void *arg1, ...);
void *ap_http_outerror_filter(void *arg1, ...);
void *ap_fini_vhost_config(void *arg1, ...);
void *ap_cleanup_scoreboard(void *arg1, ...);
void *ap_get_remote_logname(void *arg1, ...);
void *ap_add_input_filter(void *arg1, ...);
void *ap_expr_yyalloc(void *arg1, ...);
void *ap_hook_insert_network_bucket(void *arg1, ...);
void *ap_run_check_config(void *arg1, ...);
void *ap_pregfree(void *arg1, ...);
void *ap_finalize_sub_req_protocol(void *arg1, ...);
void *ap_internal_fast_redirect(void *arg1, ...);
void *ap_mpm_set_thread_stacksize(void *arg1, ...);
void *ap_init_rng(void *arg1, ...);
void *ap_send_fd(void *arg1, ...);
void *ap_expr_yyget_debug(void *arg1, ...);
void *ap_get_server_revision(void *arg1, ...);
void *ap_cfg_getc(void *arg1, ...);
void *ap_read_request(void *arg1, ...);
void *ap_is_recursion_limit_exceeded(void *arg1, ...);
void *ap_hook_open_logs(void *arg1, ...);
void *ap_hook_get_mpm_query(void *arg1, ...);
void *ap_pregcomp(void *arg1, ...);
void *ap_unixd_set_proc_mutex_perms(void *arg1, ...);
void *ap_run_insert_network_bucket(void *arg1, ...);
void *ap_expr_yy_scan_string(void *arg1, ...);
void *ap_cfg_closefile(void *arg1, ...);
void *ap_regerror(void *arg1, ...);
void *ap_create_sb_handle(void *arg1, ...);
void *ap_reserve_module_slots_directive(void *arg1, ...);
void *ap_cfg_getline(void *arg1, ...);
void *ap_update_child_status_from_indexes(void *arg1, ...);
void *ap_index_of_response(void *arg1, ...);
void *ap_expr_yyget_leng(void *arg1, ...);
void *ap_method_register(void *arg1, ...);
void *ap_hook_get_type_checker(void *arg1, ...);
void *ap_hook_get_child_status(void *arg1, ...);
void *ap_set_deprecated(void *arg1, ...);
void *ap_pool_cleanup_set_null(void *arg1, ...);
void *ap_satisfies(void *arg1, ...);
void *ap_hook_auth_checker(void *arg1, ...);
void *ap_expr_yy_flush_buffer(void *arg1, ...);
void *ap_hook_watchdog_need(void *arg1, ...);
void *ap_hook_mpm(void *arg1, ...);
void *ap_psignature(void *arg1, ...);
void *ap_method_registry_init(void *arg1, ...);
void *ap_append_pid(void *arg1, ...);
void *ap_note_basic_auth_failure(void *arg1, ...);
void *ap_hook_get_log_transaction(void *arg1, ...);
void *ap_set_sub_req_protocol(void *arg1, ...);
void *ap_set_core_module_config(void *arg1, ...);
void *ap_process_resource_config(void *arg1, ...);
void *ap_cookie_write2(void *arg1, ...);
void *ap_proc_mutex_create(void *arg1, ...);
void *ap_old_write_filter(void *arg1, ...);
void *ap_add_loaded_module(void *arg1, ...);
void *ap_run_create_connection(void *arg1, ...);
void *ap_mpm_set_coredumpdir(void *arg1, ...);
void *ap_basic_http_header(void *arg1, ...);
void *ap_set_flag_slot(void *arg1, ...);
void *ap_get_client_block(void *arg1, ...);
void *ap_mpm_pod_open(void *arg1, ...);
void *ap_hook_check_config(void *arg1, ...);
void *ap_hook_translate_name(void *arg1, ...);
void *ap_unixd_set_global_mutex_perms(void *arg1, ...);
void *ap_build_config(void *arg1, ...);
void *ap_set_accept_ranges(void *arg1, ...);
void *ap_strchr(void *arg1, ...);
void *ap_flush_conn(void *arg1, ...);
void *ap_send_mmap(void *arg1, ...);
void *ap_hook_get_watchdog_step(void *arg1, ...);
void *ap_lookup_provider(void *arg1, ...);
void *ap_allow_overrides(void *arg1, ...);
void *ap_strrchr_c(void *arg1, ...);
void *ap_getword(void *arg1, ...);
void *ap_parse_vhost_addrs(void *arg1, ...);
void *ap_remove_input_filter_byhandle(void *arg1, ...);
void *ap_escape_errorlog_item(void *arg1, ...);
void *ap_run_dirwalk_stat(void *arg1, ...);
void *ap_limit_section(void *arg1, ...);
void *ap_fflush(void *arg1, ...);
void *ap_open_piped_log_ex(void *arg1, ...);
void *ap_rflush(void *arg1, ...);
void *ap_copy_method_list(void *arg1, ...);
void *ap_run_mpm_get_name(void *arg1, ...);
void *ap_run_child_init(void *arg1, ...);
void *ap_increment_counts(void *arg1, ...);
void *ap_create_conn_config(void *arg1, ...);
void *ap_hook_check_user_id(void *arg1, ...);
void *ap_calc_scoreboard_size(void *arg1, ...);
void *ap_run_test_config(void *arg1, ...);
void *ap_varbuf_pdup(void *arg1, ...);
void *ap_find_child_by_pid(void *arg1, ...);
void *ap_some_auth_required(void *arg1, ...);
void *ap_expr_binary_op_make(void *arg1, ...);
void *ap_hook_get_get_mgmt_items(void *arg1, ...);
void *ap_no2slash(void *arg1, ...);
void *ap_hook_get_pre_connection(void *arg1, ...);
void *ap_gname2id(void *arg1, ...);
void *ap_strcasestr(void *arg1, ...);
void *ap_run_get_mgmt_items(void *arg1, ...);
void *ap_open_htaccess(void *arg1, ...);
void *ap_pcfg_openfile(void *arg1, ...);
void *ap_get_server_description(void *arg1, ...);
void *ap_scan_script_header_err_strs(void *arg1, ...);
void *ap_pregsub(void *arg1, ...);
void *ap_global_mutex_create(void *arg1, ...);
void *ap_set_module_config(void *arg1, ...);
void *ap_mpm_dump_pidfile(void *arg1, ...);
void *ap_send_interim_response(void *arg1, ...);
void *ap_expr_lookup_default(void *arg1, ...);
void *ap_find_last_token(void *arg1, ...);
void *ap_make_method_list(void *arg1, ...);
void *ap_get_mime_headers(void *arg1, ...);
void *ap_set_int_slot(void *arg1, ...);
void *ap_is_url(void *arg1, ...);
void *ap_run_insert_filter(void *arg1, ...);
void *ap_internal_redirect(void *arg1, ...);
void *ap_hook_dirwalk_stat(void *arg1, ...);
void *ap_should_client_block(void *arg1, ...);
void *ap_hook_get_header_parser(void *arg1, ...);
void *ap_is_directory(void *arg1, ...);
void *ap_hook_get_error_log(void *arg1, ...);
void *ap_register_extra_mpm_process(void *arg1, ...);
void *ap_set_mutex(void *arg1, ...);
void *ap_setup_make_content_type(void *arg1, ...);
void *ap_hook_mpm_get_name(void *arg1, ...);

